﻿Imports System
Imports System.ComponentModel.Composition
Imports Granta.HybridBase

Namespace UserModel
#Region "Source Material Data Definition"
    Public Class SourceData
        <Data("Density", "kg/m^3")>
        Public Density As Double

        <Data("Young's Modulus", "GPa")>
        Public YoungsModulus As Double

        <Data("Compressive strength", "MPa")>
        Public CompressiveStrength As Double

        <Data("Yield strength (elastic limit)", "MPa")>
        Public YieldStrength As Double

        ' When requesting price data, the unit takes the form currency/kg:
        <Data("Price", "currency/kg")>
        Public Price As Double
    End Class
#End Region

    <Export("Granta.HybridModel")>
    <Model("Advanced Model (VB)", "A more advanced example model.")>
    <Image("UserModel.model.png")>
    <ModelGroup("Examples", "UserModel.thumbnail.png")>
    Public Class ExampleModel
        <Material("Matrix", "Matrix material")>
        <Group("Matrix", 1)>
        Public matrix As SourceData

        <RecordName("Matrix", "Abbreviation used in record name")>
        Public _matrixName As String

        <Material("Reinforcement", "Reinforcement material")>
        <Group("Reinforcement", 2)>
        Public reinforcement As SourceData

        <RecordName("Reinforcement", "Abbreviation used in record name")>
        Public _reinforcementName As String

        <Variable("Reinforcement percentage", "%", "Specify the volume fraction")>
        <RangeValues(Start:=10, [End]:=70, Number:=7, Logarithmic:=False)>
        <Bounds(0, 70)>
        <Group("Reinforcement", 3)>
        Public percentage As Double

        ' If we want the user to pick from a list of options, we need to 
        ' create an enum to hold the possible choices. 
        ' Each option (enumerator) can be marked up with the [OptionItem] attribute, 
        ' which takes the description used in the UI combobox.
        Public Enum LaminateType
            <OptionItem("Unidirectional fiber")>
            Unidirectional

            <OptionItem("Quasi-isotropic fiber")>
            QuasiIsotropic
        End Enum

        ' The laminateType option that will be used in the calculations.
        ' The field must be marked as [Option] and requires a name and a description.
        <[Option]("Laminate", "Pick a laminate")>
        <Group("Reinforcement", 4)>
        Public laminate As LaminateType = LaminateType.Unidirectional

#Region "Calculations"

        ' Density is calculated by calling the method RuleOfMixture:
        <CalculatedData("Density", "kg/m^3")>
        Public Function Density() As Double
            Return RuleOfMixture(reinforcement.Density, matrix.Density)
        End Function

        ' Price - remember the unit should be in currency/kg:
        <CalculatedData("Price", "currency/kg")>
        Public Function Price() As Double
            ' Price is calculated as a rule of mixture, using the density 
            Return RuleOfMixture(reinforcement.Density * reinforcement.Price, matrix.Density * matrix.Price) / Density()
        End Function

        ' The Young's Modulus calculation changes depending on the laminateType. 
        ' The Unidirectional laminate uses the normal Rule Of Mixture.
        ' The QuasiIsotropic laminate calculates a value using the RuleOfMixture, but 
        ' with only 0.5 * the reinforcement.
        <CalculatedData("Young's modulus", "GPa")>
        Public Function YoungsModulus() As Double
            Select Case laminate
                Case LaminateType.Unidirectional
                    Return RuleOfMixture(reinforcement.YoungsModulus, matrix.YoungsModulus)

                Case LaminateType.QuasiIsotropic
                    Return RuleOfMixture(0.5 * reinforcement.YoungsModulus, matrix.YoungsModulus)

                Case Else
                    Return Double.NaN

            End Select
        End Function

        <CalculatedData("Compressive strength", "MPa")>
        Public Function CompressiveStrength() As Double
            Select Case laminate
                Case LaminateType.Unidirectional
                    Dim cs1 = RuleOfMixture(0.75 * reinforcement.CompressiveStrength, matrix.CompressiveStrength)
                    Dim cs2 = 14.0 * matrix.YieldStrength
                    Return Math.Min(cs1, cs2)

                Case LaminateType.QuasiIsotropic
                    Dim cs1 = RuleOfMixture(0.25 * reinforcement.CompressiveStrength, matrix.CompressiveStrength)
                    Dim cs2 = 14.0 * matrix.YieldStrength
                    Return Math.Min(cs1, cs2)

                Case Else
                    Return Double.NaN

            End Select
        End Function

#End Region

        ' The Rule Of Mixture implemented as a helper method, and reused in the calculations above.
        Private Function RuleOfMixture(ByVal a As Double, ByVal b As Double) As Double
            Dim f = percentage / 100
            Return f * a + (1.0 - f) * b
        End Function
    End Class
End Namespace
